#include <iostream>
#include <chrono>

int main( int          i_argc,
          char const * i_argv[] ) {
  std::cout << "running benchmark" << std::endl;

  // parse command line arguments
  if( i_argc != 3 ) {
    std::cerr << "insufficient number arguments\n"
              << "  use as:  ./build/sum SIZE N_REPETITIONS\n"
              << "  example: ./build/sum 134217728 50"
              << std::endl;
             
    return EXIT_FAILURE;
  }
  uint64_t l_size = atoll( i_argv[1] );
  uint64_t l_n_repetitions = atoll( i_argv[2] );

  // derive size in bytes and gigabytes
  uint64_t l_size_b = l_size*sizeof(uint64_t);
  double   l_size_gib = l_size_b  / (1024.0 * 1024.0 * 1024.0);

  // share config
  std::cout << "config:\n"
            << "  number of repetitions:        " << l_n_repetitions << "\n"
            << "  number of entries per vector: " << l_size << "\n"
            << "  size per vector (B):          " << l_size_b << "\n"
            << "  size per vector (GB):         " << l_size_gib
            << std::endl;

  // instrumentation vars
  std::chrono::steady_clock::time_point l_tp0, l_tp1;
  std::chrono::duration< double > l_dur;

  // alloc memory and init
  uint64_t * l_a = (uint64_t *) aligned_alloc( 4096,
                                               l_size_b );

  for( uint64_t l_en = 0; l_en < l_size; l_en++ ) {
    l_a[l_en] = 1;
  }

  // benchmark
  std::cout << "benchmarking" << std::endl;
  uint64_t l_sum = 0;
  l_tp0 = std::chrono::steady_clock::now();
  for( uint64_t l_re = 0; l_re < l_n_repetitions; l_re++ ) {
    for( uint64_t l_en = 0; l_en < l_size; l_en++ ) {
      l_sum += l_a[l_en];
    }
  }
  l_tp1 = std::chrono::steady_clock::now();

  // share results
  l_dur = std::chrono::duration_cast< std::chrono::duration< double> >( l_tp1 - l_tp0 );
  std::cout << "  error:            " << l_sum - (l_n_repetitions*l_size) << "\n"
            << "  time:             " << l_dur.count() << "\n"
            << "  bandwith (GiB/s): " << l_n_repetitions*l_size_gib / l_dur.count()
            << std::endl;

  // free memory
  free( l_a );

  return EXIT_SUCCESS;
}